# -*- coding: utf-8 -*-
import swapper
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)

Contact = swapper.load_model('organization', 'Contact')
Organization = swapper.load_model('organization', 'Organization')


# ORGANIZATION ADDRESS
class OrganizationAddressViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationAddressViewSet
    -----------

    :action list: Return the organization address list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization address.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization address list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization address instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization address instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization address instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization address instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationAddress.objects.all()
    serializer_class = serializers.OrganizationAddressSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'organization', 'country',
        ).prefetch_related(
            'address_scopes'
        )
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationAddress{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="address-type-choices")
    def address_type_choices(self, request, *args, **kwargs):
        return Response(settings.ADDRESS_ADDRESS_TYPE_CHOICES)


# EMAIL TYPOLOGY
class EmailTypologyViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    EmailTypologyViewSet
    -----------

    :action list: Return the email typology list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given email typology.
        `Profiles`: `superuser`, `staff`

    :action light: Return the email typology list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new email typology instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an email typology instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an email typology instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an email typology instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.EmailTypology.objects.all()
    serializer_class = serializers.EmailTypologySerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'EmailTypology{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an email typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an email typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PHONE TYPOLOGY
class PhoneTypologyViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    PhoneTypologyViewSet
    -----------

    :action list: Return the phone typology list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given phone typology.
        `Profiles`: `superuser`, `staff`

    :action light: Return the phone typology list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a phone typology instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PhoneTypology.objects.all()
    serializer_class = serializers.PhoneTypologySerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'PhoneTypology{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a phone typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a phone typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION TYPE
class OrganizationTypeViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationTypeViewSet
    -----------

    :action list: Return the organization type list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization type.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization type list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization type instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization type instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization type instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization type instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationType.objects.all()
    serializer_class = serializers.OrganizationTypeSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationType{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="category-choices")
    def category_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONTYPE_CATEGORY_CHOICES)


# ORGANIZATION FORM
class OrganizationFormViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationFormViewSet
    -----------

    :action list: Return the organization form list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization form.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization form list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization form instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization form instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization form instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization form instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationForm.objects.all()
    serializer_class = serializers.OrganizationFormSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationForm{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION
class OrganizationViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationViewSet
    -----------

    :action list: Return the organization list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = Organization.objects.all()
    serializer_class = serializers.OrganizationSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization_type', 'organization_form')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Organization{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# OWNER ORGANIZATION
class OwnerOrganizationViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OwnerOrganizationViewSet
    -----------

    :action list: Return the owner organization list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given owner organization.
        `Profiles`: `superuser`, `staff`

    :action light: Return the owner organization list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an owner organization instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OwnerOrganization.objects.all()
    serializer_class = serializers.OwnerOrganizationSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OwnerOrganization{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an owner organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an owner organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION EMAIL
class OrganizationEmailViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationEmailViewSet
    -----------

    :action list: Return the organization email list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization email.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization email list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization email instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization email instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization email instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization email instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationEmail.objects.all()
    serializer_class = serializers.OrganizationEmailSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationEmail{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION PHONE
class OrganizationPhoneViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationPhoneViewSet
    -----------

    :action list: Return the organization phone list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization phone.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization phone list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization phone instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationPhone.objects.all()
    serializer_class = serializers.OrganizationPhoneSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationPhone{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION SOCIAL PAGE
class OrganizationSocialPageViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationSocialPageViewSet
    -----------

    :action list: Return the organization social page list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization social page.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization social page list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization social page instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationSocialPage.objects.all()
    serializer_class = serializers.OrganizationSocialPageSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationSocialPage{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization social page instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization social page instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONSOCIALPAGE_SOCIAL_DEF)


# CONTACT ROLE
class ContactRoleViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactRoleViewSet
    -----------

    :action list: Return the contact role list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact role.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact role list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact role instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact role instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact role instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact role instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactRole.objects.all()
    serializer_class = serializers.ContactRoleSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactRole{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT TITLE
class ContactTitleViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactTitleViewSet
    -----------

    :action list: Return the contact title list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact title.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact title list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact title instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact title instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact title instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact title instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactTitle.objects.all()
    serializer_class = serializers.ContactTitleSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactTitle{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact title instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact title instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT
class ContactViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactViewSet
    -----------

    :action list: Return the contact list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = Contact.objects.all()
    serializer_class = serializers.ContactSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'user', 'organization', 'title', 'language'
        ).prefetch_related('roles')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Contact{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# OWNER CONTACT
class OwnerContactViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    OwnerContactViewSet
    -----------

    :action list: Return the owner contact list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given owner contact.
        `Profiles`: `superuser`, `staff`

    :action light: Return the owner contact list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an owner contact instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OwnerContact.objects.all()
    serializer_class = serializers.OwnerContactSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'user', 'organization', 'title', 'language'
        ).prefetch_related('roles')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OwnerContact{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an owner contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an owner contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT EMAIL
class ContactEmailViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactEmailViewSet
    -----------

    :action list: Return the contact email list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact email.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact email list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact email instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact email instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact email instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact email instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactEmail.objects.all()
    serializer_class = serializers.ContactEmailSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactEmail{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT PHONE
class ContactPhoneViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactPhoneViewSet
    -----------

    :action list: Return the contact phone list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact phone.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact phone list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact phone instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactPhone.objects.all()
    serializer_class = serializers.ContactPhoneSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactPhone{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT SOCIAL ACCOUNT
class ContactSocialAccountViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactSocialAccountViewSet
    -----------

    :action list: Return the contact social account list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact social account.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact social account list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact social account instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactSocialAccount.objects.all()
    serializer_class = serializers.ContactSocialAccountSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactSocialAccount{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact social account instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact social account instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_CONTACTSOCIALACCOUNT_SOCIAL_CHOICES)
